@@ -7,7 +7,7 @@ use File::Basename;
 use HTML::SimpleParse;
 use Symbol;
 
-$VERSION = '2.08';
+$VERSION = '2.11';
 my $debug = 0;
 
 sub handler($$) {
@@ -43,23 +43,25 @@ sub handler($$) {
         }
         $r->send_http_header;
     }
+    return if $r->header_only;
     
-    local $/ = undef;
-    $pack->new( scalar(<$fh>), $r )->output;
+    do {local $/=undef; $pack->new( scalar(<$fh>), $r )}->output;
     return OK;
 }
 
 sub new {
-    my ($pack, $text, $r) = @_;
-    
-    return bless {
-        'text' => $text,
-        '_r'   => $r,
-        'suspend' => 0,
-        'seen_true' => undef, # 1 when we've seen a true "if" in this if-chain,
-                              # 0 when we haven't, undef when we're not in an if-chain
-        'errmsg'  => "[an error occurred while processing this directive]",
-        'sizefmt' => 'abbrev',
+  my ($pack, $text, $r) = @_;
+  
+  return bless 
+    {
+     'text' => $text,
+     '_r'   => $r,
+     'suspend' => 0,
+     'seen_true' => undef, # 1 when we've seen a true "if" in this if-chain,
+                           # 0 when we haven't, undef when we're not in an if-chain
+     'errmsg'  => "[an error occurred while processing this directive]",
+     'sizefmt' => 'abbrev',
+     'timefmt' => undef, # undef means the current locale's default
     }, $pack;
 }
 
@@ -203,7 +205,7 @@ sub ssi_fsize {
 
 sub ssi_flastmod {
     my($self, $args) = @_;
-    return &_lastmod( $self->find_file($args)->filename() );
+    return &_lastmod( $self->find_file($args)->filename(), $args->{'timefmt'} || $self->{'timefmt'} );
 }
 
 sub find_file {
@@ -293,7 +295,9 @@ sub ssi_perl {
         return $self->error("sub=\"sub ...\" didn't return a reference") unless ref $subref;
     } else {             # for <!--#perl sub="package::subr" -->
         no strict('refs');
-        $subref = \&{$sub =~ /::/ ? $sub : "main::$_"};
+	$subref = (defined &{$sub} ? \&{$sub} :
+		   defined &{"${sub}::handler"} ? \&{"${sub}::handler"} : 
+		   \&{"main::$sub"});
     }
     
     $pass_r = 0 if $self->{_r} and lc $self->{_r}->dir_config('SSIPerlPass_Request') eq 'no';
@@ -315,7 +319,7 @@ sub ssi_config {
     
     $self->{'errmsg'}  =    $args->{'errmsg'}  if exists $args->{'errmsg'};
     $self->{'sizefmt'} = lc $args->{'sizefmt'} if exists $args->{'sizefmt'};
-    $self->error("'timefmt' not implemented by " . __PACKAGE__) if exists $args->{'timefmt'};
+    $self->{'timefmt'} =    $args->{'timefmt'} if exists $args->{'timefmt'};
     return '';
 }
 
@@ -416,8 +420,18 @@ sub error {
 
 sub _2main { $_[0]->is_main() ? $_[0] : $_[0]->main() }
 
-sub _lastmod($) { scalar localtime( (stat $_[0])[9] ) }
-
+sub _lastmod($;$) { # may get a timefmt as a second arg
+  if (defined $_[1]) {
+    unless (exists $INC{'Date/Format.pm'}) {
+      eval "use Date::Format";
+      warn "Can't load Date::Format: $@" if $@;
+      return if $@;
+    }
+    return strftime($_[1], [localtime( (stat $_[0])[9] )]);
+  } else {
+    return scalar localtime( (stat $_[0])[9]);
+  }
+}
 1;
 
 __END__
@@ -467,13 +481,12 @@ server-parsed html documents.  It runs under Apache's mod_perl.
 In my mind, there are two main reasons you might want to use this module:
 you can sub-class it to implement your own custom SSI directives, and/or you
 can parse the output of other mod_perl handlers, or send the SSI output
-through another handler (use Apache::Filter or Apache::OutputChain to 
-do these).
+through another handler (use Apache::Filter to do this).
 
 Each SSI directive is handled by an Apache::SSI method with the prefix
 "ssi_".  For example, <!--#printenv--> is handled by the ssi_printenv method.
 attribute=value pairs inside the SSI tags are parsed and passed to the
-method in an anonymous hash.
+method in a hash reference.
 
 =head2 SSI Directives
 
@@ -508,11 +521,23 @@ reference, or as the name of a function defined elsewhere:
  <!--#perl sub="sub { localtime() }"-->
  <!--#perl sub="time::now"-->
 
-If the 'sub' argument matches the regular expression /^\s*sub[^\w:]/, it is
-assumed to be a subroutine reference.  Otherwise it's assumed to be the name
-of a function.  In the latter case, the string "::" will be prepended to the
-function name if the name doesn't contain "::" (this forces the function to
-be in the main package, or a package you specify).
+If the 'sub' argument matches the regular expression /^\s*sub[^\w:]/,
+it is assumed to be a subroutine reference.  Otherwise it's assumed to
+be the name of a function.  In the latter case, the string "main::"
+will be prepended to the function name if the name doesn't contain
+"::" (this forces the function to be in the main package, or a package
+you specify).  Note that it's a pretty bad idea to put your code in
+the main package, so I only halfheartedly endorse this feature.
+
+In general, it will be slower to use anonymous subroutines, because
+each one has to be eval()'ed and there is no caching.  For best
+results, pre-load any code you need in the parent process, then call
+it by name.
+
+If you're calling a subroutine like "&Package::SubPack::handler", you
+can omit the "handler" portion, making your directive like this:
+
+ <!--#perl sub="Package::Subpack"-->
 
 If you want to supply a list of arguments to the function, you use either
 the "arg" or the "args" parameter:
